import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { BackendError } from 'app/shared/models/backend-error';
import { BaseService } from 'app/shared/services/base.service';
import { environment } from 'environments/environment';
import { Observable } from 'rxjs';
import { catchError, map } from 'rxjs/operators';
import { PublicConfig } from '../models/public-config';

@Injectable({
    providedIn: 'root'
})
export class ConfigService extends BaseService {

    constructor(private http: HttpClient) {
        super();
    }

    getConfig(): Observable<PublicConfig> {
        return this.http.get<PublicConfig>(`${environment.backendUrl}/config`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(conf => new PublicConfig(conf))
        );
    }
}