import { Component, Input, OnInit } from '@angular/core';
import { Exercise } from 'app/features/exercises/models/exercise';
import { ExerciseService } from 'app/features/exercises/services/exercise.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { takeUntil } from 'rxjs/operators';
import { Course } from '../../models/course';

@Component({
  selector: 'ngx-last-exercises-list',
  templateUrl: './last-exercises-list.component.html',
  styleUrls: ['./last-exercises-list.component.scss']
})
export class LastExercisesListComponent extends BaseComponent implements OnInit {
  @Input() course: Course;
  @Input() compactMode: boolean;
  exercises: Exercise[] = [];
  loading: boolean;
  error: BackendError;

  constructor(
    private exerciseService: ExerciseService
  ) { 
    super();
  }

  ngOnInit(): void {
    this.loading = true;
    
    this.exerciseService.getForCourse(this.course.id)
      .pipe(
        takeUntil(this.unsubscribe),
      )
      .subscribe((exercises) => {
        let visibleExercises = exercises.filter(e => e.visible);
        visibleExercises.sort((a, b) => b.id - a.id);
        this.exercises = visibleExercises.slice(0, Math.min(3, visibleExercises.length));
        
        this.loading = false;
      }, (error) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

}
