import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { takeUntil } from 'rxjs/operators';
import { Exercise } from '../../models/exercise';
import { Submission } from '../../models/submission';
import { TaskInstance } from '../../models/task-instance';
import { ExerciseService } from '../../services/exercise.service';
import { SubmissionService } from '../../services/submission.service';

@Component({
  selector: 'ngx-exercise-tasks-list',
  templateUrl: './exercise-tasks-list.component.html',
  styleUrls: ['./exercise-tasks-list.component.scss']
})
export class ExerciseTasksListComponent extends BaseComponent implements OnInit {
  @Input() exercise: Exercise;
  @Input() submission: Submission | null;
  @Input() showAttempts: boolean = true;
  @Input() compactMode: boolean = false;
  @Input() showDescription: boolean = false;
  @Input() showLanguage: boolean = true;
  @Input() showPoints: boolean = false;
  @Input() detailLinkSupplier: (ti: TaskInstance) => any = (ti) => null;
  @Input() detailLinkParamsSupplier: (ti: TaskInstance) => any = (ti) => null;

  _selectedTaskInstance: TaskInstance | null;
  @Output() taskSelected: EventEmitter<TaskInstance> = new EventEmitter<TaskInstance>();
  @Output() tasksLoaded: EventEmitter<TaskInstance[]> = new EventEmitter<TaskInstance[]>();

  @Input() set selectedTaskInstance(val: TaskInstance | null) {
    this._selectedTaskInstance = val;
  }

  get selectedTaskInstance(): TaskInstance | null {
    return this._selectedTaskInstance;
  }

  loading: boolean = true;
  error: BackendError;
  taskInstances: TaskInstance[] = [];

  constructor(
    private exerciseService: ExerciseService,
    private submissionService: SubmissionService,
  ) { 
    super();
  }

  ngOnInit(): void {
    this.exerciseService.getTaskInstances(this.exercise.id)
    .pipe(
      takeUntil(this.unsubscribe),
    )
    .subscribe(taskInstances => {
      this.taskInstances = taskInstances;
      this.loading = false;
      this.tasksLoaded.emit(this.taskInstances);
    }, error => {
      if (BackendError.isBackendError(error)) {
        this.error = error;
        this.loading = false;
      } else {
        throw error;
      }
    });
  }

  selectTask(taskInstance: TaskInstance) {
    this.taskSelected.emit(taskInstance);
  }
}
