import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppStateService } from 'app/core/services/app.state.service';
import { Course } from 'app/features/courses/models/course';
import { CourseService } from 'app/features/courses/services/course.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { ParamHelper } from 'app/shared/helper/param.helper';
import { BackendError } from 'app/shared/models/backend-error';
import { takeUntil, flatMap } from 'rxjs/operators';

@Component({
  selector: 'ngx-new-exercise',
  templateUrl: './new-exercise.component.html',
  styleUrls: ['./new-exercise.component.scss']
})
export class NewExerciseComponent extends BaseComponent implements OnInit {
  loading: boolean = true;
  course: Course;

  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diesen Kurs konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];

  constructor(
    private courseService: CourseService,
    private appStateService: AppStateService,
    private route: ActivatedRoute) {
    super();
  }

  buildBreadcrumbs(course: Course) {
    this.breadcrumbs = [
      {
        id: 'exercises',
        params: {
          course_id: course.id
        },
      },
      {
        id: 'newExercise',
        params: {
          course_id: course.id
        },
      },
    ];
  }

  ngOnInit(): void {
    this.route.paramMap
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(params => {
          let courseId = ParamHelper.parseIdOrThrow(params, "course_id");
          return this.courseService.getById(courseId);
        }),
      ).subscribe(course => {
        this.course = course;
        this.appStateService.setCourse(course);
        this.buildBreadcrumbs(course);

        this.loading = false;
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }
}
