import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { forkJoin, Observable, of } from "rxjs";
import { catchError, flatMap, map } from "rxjs/operators";
import { UpdatePassedDto } from "../dto/update-passed.dto";
import { UpdatePointsDto } from "../dto/update-points.dto";
import { Submission } from "../models/submission";
import { BackendError } from "app/shared/models/backend-error";

export interface SubmissionFilter {
    exerciseId?: number;
    unitId?: number;
    userId?: number;
}

@Injectable({
    providedIn: 'root'
})
export class SubmissionService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    getById(id: number): Observable<Submission> {
        return this.http.get<Submission>(`${environment.backendUrl}/submission/${id}`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(solution => new Submission(solution))
        );
    }

    getSubmissions(filter: SubmissionFilter): Observable<Submission[]> {
        var params = new HttpParams();
        if(filter.exerciseId != null) {
            params = params.set('exerciseId', filter.exerciseId);
        }

        if(filter.unitId != null) {
            params = params.set('unitId', filter.unitId);
        }

        if(filter.userId != null) {
            params = params.set('userId', filter.userId);
        }

        return this.http.get<Submission[]>(`${environment.backendUrl}/submission`, {
            params: params
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(submissions => submissions.map(submission => new Submission(submission)))
        )
    }

    updatePoints(submissionId: number, dto: UpdatePointsDto): Observable<Submission> {
        return this.http.post<Submission>(`${environment.backendUrl}/submission/${submissionId}/points`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(submission => new Submission(submission))
        );
    }

    updatePassed(submissionId: number, dto: UpdatePassedDto): Observable<Submission> {
        return this.http.post<Submission>(`${environment.backendUrl}/submission/${submissionId}/passed`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(submission => new Submission(submission))
        );
    }

  
}