export namespace RefFactory {
    export function create(ref: any) {
        if(ref.type == 'video') {
            return new VideoRef(ref);
        } else if(ref.type == 'script') {
            return new ScriptRef(ref);
        } else if(ref.type == 'cardelli') {
            return new CardelliRef(ref);
        } 

        throw `ref unknown: ${ref}`;
    }
}

export class Ref {
    title: string;

    constructor(dto: any) {
        this.title = dto.title;
    }

    icon(): string {
        throw "unknown item, icon() not supported";
    }

    url(): string {
        throw "unknown item, url() not supported";
    }

    subtitle() {
        throw "unknown item, subtitle() not supported";
    }
}

export class VideoRef extends Ref {
    _url: string;

    constructor(dto: any) {
        super(dto);
        this._url = dto.url;
    }

    icon(): string {
        return 'circle-play';
    }

    url(): string {
        return this._url;
    }

    subtitle() {
        return "Video";
    }
}

export class ScriptRef extends Ref {
    endPage: number;
    startPage: number;
    _url: string;

    constructor(dto: any) {
        super(dto);
        this._url = dto.url;
        this.endPage = dto['end-page'];
        this.startPage = dto['start-page'];
    }

    icon(): string {
        return 'bookmark';
    }

    url(): string {
        return this._url;
    }

    subtitle() {
        return `S. ${this.startPage}-${this.endPage}`;
    }
}

export class CardelliRef extends Ref {
    courseId: string;

    constructor(dto: any) {
        super(dto);
        this.courseId = dto.courseId;
    }

    icon(): string {
        return 'message';
    }

    url(): string {
        return `https://procomp.cs.hs-rm.de/subatoItemServer/cardelliWeb/?course=${this.courseId}`;
    }

    subtitle() {
        return "Cardelli-Kurs";
    }
}
