import { Component, OnInit } from '@angular/core';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { takeUntil } from 'rxjs/operators';
import { Misconception } from '../../models/misconception';
import { MisconceptionService } from '../../services/misconception.service';

@Component({
  selector: 'ngx-misconceptions',
  templateUrl: './misconceptions.component.html',
  styleUrls: ['./misconceptions.component.scss']
})
export class MisconceptionsComponent extends BaseComponent implements OnInit {
  loading: boolean;
  misconceptions: Misconception[] = [];
  error: BackendError;


  constructor(
    private misconceptionService: MisconceptionService,
  ) {
    super();
  }

  ngOnInit(): void {
    this.loading = true;
    this.misconceptionService.getAll()
    .pipe(
      takeUntil(this.unsubscribe)
    ).subscribe((misconceptions) => {
      this.misconceptions = misconceptions;
      this.loading = false;
    }, error => {
      if (BackendError.isBackendError(error)) {
        this.error = error;
        this.loading = false;
      } else {
        throw error;
      }
  }); 
  }

}
