package de.hsrm.sls.subato.aor.term;

import de.hsrm.sls.subato.modules.term.TermRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;

// gradle bootRun --args='import:term /home/fischer/Desktop/term-data'
@Component
public class TermDataImportRunner implements CommandLineRunner {
    @Autowired
    TermDataImportManager termDataImportManager;

    @Autowired
    TermRepository termRepository;

    Logger logger = LoggerFactory.getLogger(TermDataImportRunner.class);

    @Override
    public void run(String... args) {
        if (args.length < 1 || !args[0].equals("import:term")) return;

        var filePath = args[1];
        var coreDataDir = new File(filePath);
        if(!coreDataDir.exists()) {
            logger.error("Directory %s does not exist".formatted(filePath));
            System.exit(0);
        }

        logger.info("Importing term data from files in directory %s...".formatted(filePath));

        for(var file : coreDataDir.listFiles()) {
            try {
                var json = Files.readString(file.toPath(), StandardCharsets.UTF_8);
                termDataImportManager.importFromJson(json);
                logger.info("Finished importing term data %s".formatted(file.toPath().toString()));
            } catch (IOException e) {
                logger.error("File %s not readable".formatted(file.toPath()), e);
            }
        }

        logger.info("Finished importing term data");
        System.exit(0);
    }
}
