package de.hsrm.sls.subato.attendances.attendance;

import de.hsrm.sls.subato.attendances.attendance.dto.*;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.attendance.AttendancePermission;
import de.hsrm.sls.subato.courses.course.CourseNotFoundException;
import de.hsrm.sls.subato.courses.course.CourseService;
import de.hsrm.sls.subato.courses.event.CourseUnitEventNotFoundException;
import de.hsrm.sls.subato.courses.event.CourseUnitEventService;
import de.hsrm.sls.subato.courses.unit.CourseUnitNotFoundException;
import de.hsrm.sls.subato.courses.unit.CourseUnitService;
import de.hsrm.sls.subato.user.UserNotFoundException;
import de.hsrm.sls.subato.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;

import static org.springframework.http.HttpStatus.BAD_REQUEST;

@RestController
@Tag(name = "Anwesenheitserfassung")
public class AttendanceController {
    @Autowired
    private AttendanceMapper attendanceMapper;

    @Autowired
    private AttendanceService attendanceService;

    @Autowired
    private UserService userService;

    @Autowired
    private CourseUnitEventService courseUnitEventService;

    @Autowired
    private PolicyEvaluator policy;


    @PutMapping("/attendance")
    AttendanceDto createAttendance(@RequestBody CreateAttendanceDto dto) {
        try {
            var event = courseUnitEventService.findById(dto.getEventId());
            var user = userService.findById(dto.getUserId());

            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(event, AttendancePermission.ATTENDANCE_UPDATE));

            var attendance = attendanceService.createAttendance(user, event);
            return attendanceMapper.map(attendance);
        } catch(CourseUnitEventNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, "Termin existiert nicht");
        } catch(UserNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, "Benutzer existiert nicht");
        }
    }

    @DeleteMapping("/attendance/user/{userId}/event/{eventId}")
    void deleteAttendance(@PathVariable Long userId, @PathVariable Long eventId) {
        try {
            var event = courseUnitEventService.findById(eventId);
            var user = userService.findById(userId);

            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(event, AttendancePermission.ATTENDANCE_UPDATE));

            attendanceService.deleteAttendance(user, event);
        } catch(AttendanceNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, AttendanceNotFoundException.REASON);
        }
    }
}
