package de.hsrm.sls.subato.auth.policy;

import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import org.springframework.security.core.context.SecurityContextHolder;

import java.util.HashMap;
import java.util.Map;

public class CheckPolicyRequest {
    SubatoPrincipal subject;
    Object obj;
    Object permission;
    Map<String, Object> environment;

    public CheckPolicyRequest(SubatoPrincipal subject, Object obj, Object permission, Map<String, Object> environment) {
        this.subject = subject;
        this.obj = obj;
        this.permission = permission;
        this.environment = environment;
    }

    public static CheckPolicyRequest fromPrincipal(Object obj, Object permission, Map<String, Object> environment) {
        var principal = SecurityContextHolder.getContext().getAuthentication();
        var user = principal.getPrincipal() instanceof SubatoPrincipal ? (SubatoPrincipal) principal.getPrincipal() : null;
        return new CheckPolicyRequest(user, obj, permission, environment);
    }

    public static CheckPolicyRequest fromPrincipal(Object obj, Object permission) {
        return fromPrincipal(obj, permission, new HashMap<>());
    }

    public boolean authenticated() {
        return subject != null;
    }

    public boolean unauthenticated() {
        return !authenticated();
    }

    public boolean matches(String permission) {
        return this.permission != null && this.permission.equals(permission);
    }
    public SubatoPrincipal getSubject() {
        return subject;
    }

    public Object getObj() {
        return obj;
    }

    public Object getPermission() {
        return permission;
    }

    public Map<String, Object> getEnvironment() {
        return environment;
    }
}
