package de.hsrm.sls.subato.courses.course;

import com.google.common.base.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Responsible for returning an instance of the configured {@link CourseSupplement} implementation dynamically at runtime
 */
@Service
public class CourseSupplementFactory {
    private static Logger logger = LoggerFactory.getLogger(CourseSupplementFactory.class);
    private static CourseSupplement courseSupplement = null;

    @Autowired
    private CourseSupplementFactory(
            List<CourseSupplement> services,
            @Value("${subato.course-supplement}") String supplementIdentifier
    ) {
        var availableSupplements = String.join(", ", services.stream().map(impl -> impl.getIdentifier()).toList());

        if(!Strings.isNullOrEmpty(supplementIdentifier)) {
            courseSupplement = services.stream().filter(impl -> impl.getIdentifier().equals(supplementIdentifier)).findFirst().orElse(null);
            if (courseSupplement == null) {
                throw new RuntimeException("CourseSupplement with identifier %s not found, did you override the subato.course-supplement config property? Available handlers: %s".formatted(supplementIdentifier, availableSupplements));
            }

            logger.info("Active CourseSupplement: %s (%s)".formatted(courseSupplement.getClass().getName(), courseSupplement.getIdentifier()));
        }
    }

    public static CourseSupplement create() {
        return courseSupplement;
    }
}
