package de.hsrm.sls.subato.exercises.assessment;

import de.hsrm.sls.subato.courses.unit.CourseUnit;
import de.hsrm.sls.subato.courses.unit.CourseUnitService;
import de.hsrm.sls.subato.courses.unit.CourseUnitType;
import de.hsrm.sls.subato.exercises.exercise.AssessmentMode;
import de.hsrm.sls.subato.exercises.exercise.Exercise;
import de.hsrm.sls.subato.exercises.submission.Submission;
import de.hsrm.sls.subato.exercises.submission.SubmissionFilter;
import de.hsrm.sls.subato.exercises.submission.SubmissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class AssessmentProgressService {
    @Autowired
    private SubmissionService submissionService;

    @Autowired
    private CourseUnitService courseUnitService;

    public UnitAssessmentProgress getProgressForExerciseAndUnit(Exercise exercise, CourseUnit unit) {
        var filter = new SubmissionFilter().exerciseId(exercise.getId()).unitId(unit.getId());
        var submissions = submissionService.findAll(filter);
        submissions.sort(Comparator.comparing(submission -> submission.getSubmittedBy().getName()));

        var unitProgress = new UnitAssessmentProgress();
        unitProgress.setUnit(unit);
        unitProgress.setProgress(createAssessmentProgress(exercise, submissions));

        var nextSubmission = submissions.stream().filter(s -> s.getPassed() == null).findFirst().orElse(
                submissions.stream().findFirst().orElse(null)
        );

        if(nextSubmission != null) {
            unitProgress.setNextSubmissionId(nextSubmission.getId());
        }

        return unitProgress;
    }

    public List<UnitAssessmentProgress> getProgressForExercise(Exercise exercise) {
        var units = courseUnitService.findByCourse(exercise.getCourse()).stream()
                .filter(u -> u.getType() == CourseUnitType.Lab || u.getType() == CourseUnitType.Misc)
                .collect(Collectors.toList());

        var unitProgresses = new ArrayList<UnitAssessmentProgress>();

        for(var unit : units) {
            var unitProgress = getProgressForExerciseAndUnit(exercise, unit);
            unitProgresses.add(unitProgress);
        }

        return unitProgresses;
    }

    private static AssessmentProgress createAssessmentProgress(Exercise exercise, List<Submission> submissions) {
        if(exercise.getAssessmentMode() == AssessmentMode.Binary) {
            var progress = new BinaryAssessmentProgress();
            progress.setPassedTotal(submissions.stream().filter(s -> s.getPassed() != null && s.getPassed()).count());
            progress.setNotPassedTotal(submissions.stream().filter(s -> s.getPassed() != null && !s.getPassed()).count());
            progress.setUndecidedTotal(submissions.stream().filter(s -> s.getPassed() == null).count());
            progress.setTotal(submissions.size());
            return progress;
        }

        if(exercise.getAssessmentMode() == AssessmentMode.Points) {
            var progress = new PointsAssessmentProgress();
            progress.setFullyRatedTotal(submissions.stream().filter(s -> s.getPointRatingCount() == s.getExercise().getTaskInstancesCount()).count());
            progress.setTotal(submissions.size());
            return progress;
        }

        if(exercise.getAssessmentMode() == null) {
            var progress = new NullAssessmentProgress();
            progress.setTotal(submissions.size());
            return progress;
        }

        return null;
    }
}
