package de.hsrm.sls.subato.solutions.evaluation.result;

import com.fasterxml.jackson.annotation.JsonIgnore;

import jakarta.persistence.*;
import org.hibernate.annotations.JdbcTypeCode;
import org.hibernate.type.SqlTypes;

@Entity
@Table(name = "test_case_result")
public class TestCaseResult {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    /**
     * name of the test case method
     */
    private String name;

    /**
     * name of the test class containing the test case
     */
    private String className;

    /**
     * message of the assert that lead to the test failure, if any
     */
    @JdbcTypeCode(SqlTypes.LONGVARCHAR)
    private String message;

    /**
     * detailed message which extends {@link #message}, may include stack traces or other information
     */
    @JdbcTypeCode(SqlTypes.LONGVARCHAR)
    private String text;

    /**
     * fully qualified name of the test case in the form *className*(*name*)
     * ?
     */
    private String type;

    @JsonIgnore
    @ManyToOne
    private EvalResult evalResult;

    public long getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public String getClassName() {
        return className;
    }

    public String getText() {
        return text;
    }

    public String getMessage() {
        return message;
    }

    public String getType() {
        return type;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public void setText(String text) {
        this.text = text;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public void setType(String type) {
        this.type = type;
    }

    public EvalResult getEvalResult() {
        return evalResult;
    }

    public void setEvalResult(EvalResult evalResult) {
        this.evalResult = evalResult;
    }
}
