package de.hsrm.sls.subato.solutions.solution;

import de.hsrm.sls.subato.shared.config.SubatoConfig;
import de.hsrm.sls.subato.shared.fs.FileContent;
import de.hsrm.sls.subato.shared.fs.FileTreeCache;
import de.hsrm.sls.subato.shared.fs.FileTree;
import de.hsrm.sls.subato.shared.helper.ZipHelper;
import net.lingala.zip4j.model.ExcludeFileFilter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.FileSystemUtils;

import java.io.File;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.UUID;

@Service
public class SolutionContentRepository {
    @Autowired
    private SubatoConfig subatoConfig;

    @Autowired
    private FileTreeCache fileTreeCache;

    private Path getSolutionPath(Solution solution) {
        return Paths.get(subatoConfig.getDataDir())
                .resolve("solution")
                .resolve(String.valueOf(solution.getSubmittedAt().getYear()))
                .resolve(String.valueOf(solution.getSubmittedAt().getMonthValue()))
                .resolve(String.valueOf(solution.getSubmittedAt().getDayOfMonth()))
                .resolve(solution.getId().toString());
    }

    public void store(FileContent content, Solution solution) {
        var task = solution.getTask();
        content.saveTo(getSolutionPath(solution).toFile(), task.isExtractArchives());
    }

    public FileTree getTree(Solution solution) {
        var spec = solution.getTask().getSpec();
        return fileTreeCache.getTree(getSolutionPath(solution), spec.getMeta().getMimeMappings());
    }

    /**
     * gets the content of a specific file in the solution
     * @param solution solution from where to get the file
     * @param uuid identifier for the file to get,
     *             retrieved through the {@link de.hsrm.sls.subato.shared.fs.FileTree}
     *             of the solution
     */
    public File getFile(Solution solution, UUID uuid) {
        var solutionPath = getSolutionPath(solution);
        var spec = solution.getTask().getSpecCache().getSpec();

        var solutionTree = fileTreeCache.getTree(solutionPath, spec.getMeta().getMimeMappings());
        var tree = solutionTree.find(i -> i.getUuid().equals(uuid));
        var file = tree.getFile(solutionPath);

        return file;
    }

    public File getSolutionAsZip(Solution solution) {
        var root = getSolutionPath(solution);
        ExcludeFileFilter excludeFilter = file -> file.toPath().equals(fileTreeCache.getTreeIndexPath(root));
        return ZipHelper.zip(root.toFile(), false, excludeFilter);
    }

    public void delete(Solution solution) {
        var solutionPath = getSolutionPath(solution);
        FileSystemUtils.deleteRecursively(solutionPath.toFile());
    }
}
