package de.hsrm.sls.subato.tasks.imp;

import de.hsrm.sls.subato.tasks.pool.TaskPoolNotFoundException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

import java.nio.file.Path;

/**
 * Responsible for manually importing tasks (represented as subdirs in the file system)
 * from a certain directory (root_dir)
 * into a task pool (task_pool_id).
 *
 * for starting the import, run:
 * gradle bootRun --args='import:tasks %root_dir% %task_pool_id%'
 *
 * example:
 * gradle bootRun --args='import:tasks /home/fischer/Desktop/TaskImport/OOSE_AI 1'
 * gradle bootRun --args='import:tasks /home/fischer/Desktop/TaskImport/PMT_AI 2'
 */
@Component
public class ImportTasksRunner implements CommandLineRunner {
    @Autowired
    TaskImportService taskImportService;

    private static Logger logger = LoggerFactory.getLogger(ImportTasksRunner.class);

    @Override
    public void run(String... args) {
        if (args.length < 1 || !args[0].equals("import:tasks")) return;
        var rootDir = Path.of(args[1]).toFile();
        var poolId = Long.parseLong(args[2]);

        try {
            taskImportService.importTasks(poolId, rootDir);
        } catch(TaskPoolNotFoundException ex) {
            logger.error("No pool found with id %s".formatted(ex.getId()));
        } catch (TaskImportException ex) {
            logger.error("Error while importing tasks", ex);
        }

        System.exit(0);
    }
}
