package de.hsrm.sls.subato.tasks.pool.sync;

import de.hsrm.sls.subato.tasks.pool.TaskPoolService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

@Component
public class TaskPoolSyncJob {
    @Autowired
    TaskPoolSyncService syncService;

    @Autowired
    TaskPoolService taskPoolService;

    private static Logger logger = LoggerFactory.getLogger(TaskPoolSyncJob.class);

    @Scheduled(cron = "#{ '${subato.task-pool-sync.enabled}' ? '${subato.task-pool-sync.cron}' : '-' }")
    public void run() {
        logger.info("Running Task Pool sync job");

        try {
            var pools = taskPoolService.getAllWithRepository();
            for(var pool : pools) {
                syncService.syncPool(pool, false);
            }

            logger.info("Finished Task Pool sync job");
        } catch(Exception e) {
            logger.error("Running Task Pool sync job failed", e);
        }
    }
}
