package de.hsrm.sls.subato.tasks.stef.parser;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * If math expressions (Tex) were entered in the WYSIWYG editor from subato v1, html tags (like <br>) are inserted.
 * We need to remove these because tex renderers (such as Katex) cannot parse the poluted expressions.
 */
public class TexSanitizer {
    public String removeHtmlInTexExpressions(String html) {
        String newHtml = html;

        // Remove HTML tags enclosed in $$...$$
        var p1 = Pattern.compile("\\${2}(.+?)\\${2}");
        newHtml = p1.matcher(newHtml).replaceAll(match -> Matcher.quoteReplacement("$$" + replaceHtmlTags(match.group(1)) + "$$"));

        // Remove HTML tags enclosed in $...$ (does not match $$...$$)
        // wtf
        var p2 = Pattern.compile("(?<!\\$)\\$(?!\\$)(.*?)(?<!\\$)\\$(?!\\$)");
        newHtml = p2.matcher(newHtml).replaceAll(match -> {
            return Matcher.quoteReplacement("$" + replaceHtmlTags(match.group(1)) + "$");
        });

        // Remove HTML tags enclosed in \[\[...\]\]
        // wtf wtf wtf
        var p3 = Pattern.compile("(?:\\\\\\[){2}(.+?)(?:\\\\\\]){2}");
        newHtml = p3.matcher(newHtml).replaceAll(match -> "\\\\[\\\\[" + replaceHtmlTags(match.group(1)) + "\\\\]\\\\]");

        // Remove HTML tags enclosed in \[...\] (does not match \[\[...\]\])
        // wtf^9000 - sorry
        var p4 = Pattern.compile("(?<!(?:\\\\\\[))\\\\\\[(?!(?:\\\\\\[))(.*?)(?<!(?:\\\\\\]))\\\\\\](?!(?:\\\\\\]))");
        newHtml = p4.matcher(newHtml).replaceAll(match -> {
            return "\\\\[" + replaceHtmlTags(match.group(1)) + "\\\\]";
        });

        return newHtml;
    }

    private String replaceHtmlTags(String html) {
        return html.replaceAll("</?[^>]+(>|$)", "");
    }
}
