package de.hsrm.sls.subato.tasks.task;

import de.hsrm.sls.subato.tasks.imp.log.LogType;
import de.hsrm.sls.subato.tasks.imp.log.TaskImportLogService;
import de.hsrm.sls.subato.tasks.stef.cache.StefSpecCache;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.stef.parser.SpecParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class TaskVerificationService {
    private static Logger logger = LoggerFactory.getLogger(TaskVerificationService.class);

    @Autowired
    TaskService taskService;

    @Autowired
    TaskRepository taskRepository;

    @Autowired
    TaskContentRepository taskContentRepository;

    @Autowired
    TaskImportLogService importLogService;

    @Autowired
    private SpecParser specParser;

    public void verifyTasks() {
        logger.info("Started verifying tasks");

        var tasks = taskService.getAll(new TaskFilter());
        for(var task : tasks) {
            if(task.isArchived() || task.hasValidSpec()) {
                continue;
            }

            logger.debug("Rebuilding cache for task %s".formatted(task.getId()));

            try {

                var taskPath = taskContentRepository.getTaskPath(task);
                var spec = specParser.parseSpec(taskPath.toString());
                task.setSpecCache(new StefSpecCache(StefSpecCache.SERIALIZATION_VERSION, spec));
                task = taskRepository.save(task);
            } catch(InvalidSpecException e) {
                logger.debug("Spec for task %s is invalid: %s".formatted(task.getId(), e.getMessage()));

                taskService.archive(task);
                logger.debug("Archived task %s".formatted(task.getId()));

                if(task.getPool() != null) {
                    importLogService.log(task.getPool(), LogType.PROBLEM, "Verifikation von %s fehlgeschlagen: %s. %s".formatted(task.getIdName(), InvalidSpecException.REASON, e.getMessage()));
                }
            }
        }

        logger.info("Verification of tasks finished");
    }
}
